//-----------------------------------------------------------------------------
// File:     UserForceATPTorqueSpring.h
// Class:    UserForceATPTorqueSpring
// Parent:   GeneralForceElements
// Children: None
// Purpose:  Applies a torque resulting from ATP hydrolysis
// Author:   Mary Elting - May 30, 2007
//-----------------------------------------------------------------------------
#ifndef __USERFORCEATPTORQUESPRING_H__
#define __USERFORCEATPTORQUESPRING_H__
//-----------------------------------------------------------------------------
#include "StandardCppHeadersAndNamespace.h"
#include "SimTKsimbody.h"
using namespace SimTK;

//--------------------------------------------------------------------------
// User-defined classes for adding forces/torques are constructed as follows:
// 1. Create a constructor with whatever arguments make sense for the force or torque and copy the arguments into class data.
// 2. Create a clone method (all clone methods are identical except for the class name appearing after "new")
// 3. Create a calc  method (the arguments and return type for all calc methods are identical).
//    The code in the calc method is specific to the calculation of force or torque.
// Note: The set of all forces is replaced by an equivalent set, consisting of a torque
//       that is equal to the moment of the forces about the body's origin together
//       with the resultant of the forces applied at the body's origin.
//--------------------------------------------------------------------------
class UserForceATPTorqueSpring : public GeneralForceElements::UserForce 
{
public:
   // Constructor is explicit
   explicit UserForceATPTorqueSpring( BodyId bodyIdA, Real naturalAngle, Real k, Real damping  )
   { 
      myBodyIdForApplyingForce = bodyIdA;
      //angle with the vertical (y) axis
      myNaturalAngle =  naturalAngle;
      myK = k;
      myDamping = damping;
   } 

   // The clone method is used internally by Simbody (required by virtual parent class)
   UserForce*  clone() const  { return new UserForceATPTorqueSpring(*this); }

   // The calc method is where forces or torques are calculated (required by virtual parent class)
   void  calc( const MatterSubsystem& matter,              // Input information (matter)
               const State&           state,               // Input information (current state)
               Vector_<SpatialVec>&   bodyForces,          // Forces and torques on bodies
               Vector_<Vec3>&         particleForces,      // Forces on particles (currently unused)
               Vector&                mobilityForces,      // Generalized forces 
               Real&                  pe ) const           // For forces with a potential energy
    {
       
       
       //Get the body's orientation
       Vec3 yVec(0, 1, 0);

       Vec3 orientation = matter.calcBodyVectorInBody(state, myBodyIdForApplyingForce, yVec, GroundId);
       Real currentAngle = atan(orientation[0]/orientation[1]);
       Real zTorque = -myK*(myNaturalAngle-currentAngle);
       Vec3 angularVel = matter.calcBodyAngularVelocityInBody(state, myBodyIdForApplyingForce, GroundId);
       Real angVel3 = angularVel[2];
       Real zDamping = -myDamping*angularVel[2]; // put in some damping relative to z torque
       zTorque = zTorque + zDamping;

        // Get the proper memory location to increment the force and/or torque.
        // bodiesForces is a Vec6 whose elements are two Vec3.
        // The elements of the first  Vec3 are Tx, Ty, Tz  (expressed in the ground's "x,y,z").
        // The elements of the second Vec3 are Fx, Fy, Fz  (expressed in the ground's "x,y,z").
        SpatialVec& bodiesForces = bodyForces[ myBodyIdForApplyingForce ];
        Vec3& torqueSum = bodiesForces[0];
        Vec3& forceSum  = bodiesForces[1];

        // Increment the sum of all forces on this body (other force subsystems may also add forces/torque)
        //torqueSum[0] += xTorque;       // Increment torque in the ground's x-direction.
        //torqueSum[1] += yTorque;       // Increment torque in the ground's y-direction.
        Real origZTorque = torqueSum[2];
        torqueSum[2] += zTorque;       // Increment torque in the ground's z-direction.
        // forceSum[0]  += xForce;       // Increment force  in the ground's x-direction.
        // forceSum[1]  += yForce;     // Increment force  in the ground's y-direction.
        // forceSum[2]  += zForce;       // Increment force  in the ground's z-direction.
    }

private:
    BodyId  myBodyIdForApplyingForce;
    Real    myNaturalAngle;
    Real    myK;
    Real    myDamping;
};
//-----------------------------------------------------------------------------
#endif /* __USERFORCELINEARSPRING_H__ */
//-----------------------------------------------------------------------------
